import * as v from 'valibot';

declare const _URL: {
    new (url: string | URL, base?: string | URL): URL;
    prototype: URL;
    canParse(url: string | URL, base?: string | URL): boolean;
    createObjectURL(obj: Blob | MediaSource): string;
    parse(url: string | URL, base?: string | URL): URL | null;
    revokeObjectURL(url: string): void;
};
declare const _URLSearchParams: {
    new (init?: string[][] | Record<string, string> | string | URLSearchParams): URLSearchParams;
    prototype: URLSearchParams;
};
type Fetch = typeof fetch;
type FetchResponse = Response;
declare const _Headers: typeof globalThis.Headers;
type FetchHeaders = globalThis.Headers;
type FetchRequestInit = RequestInit;

declare class JsonParseError extends Error {
    constructor(message: string, jsonString: string);
}

declare class ValidationError<Schema extends v.BaseSchema<any, any, any> = v.BaseSchema<any, any, any>> extends Error {
    readonly valibotIssues: Array<v.InferIssue<Schema>>;
    constructor(message: string, valibotIssues?: Array<v.InferIssue<Schema>>);
}

declare class InvalidFetchResponseError extends Error {
    readonly textResponse: string;
    readonly response: FetchResponse;
    constructor(message: string, textResponse: string, response: FetchResponse);
}

/**
 * Get the time in seconds since epoch for a date.
 * If date is not provided the current time will be used.
 */
declare function dateToSeconds(date?: Date): number;
declare function addSecondsToDate(date: Date, seconds: number): Date;

declare function decodeUtf8String(string: string): Uint8Array;
declare function encodeToUtf8String(data: Uint8Array): string;
/**
 * Also supports base64 url
 */
declare function decodeBase64(base64: string): Uint8Array;
declare function encodeToBase64(data: Uint8Array | string): string;
declare function encodeToBase64Url(data: Uint8Array | string): string;

type BaseSchema = v.BaseSchema<any, any, any>;
type InferOutputUnion<T extends readonly any[]> = {
    [K in keyof T]: v.InferOutput<T[K]>;
}[number];
declare function stringToJsonWithErrorHandling(string: string, errorMessage?: string): any;
declare function parseWithErrorHandling<Schema extends BaseSchema>(schema: Schema, data: unknown, customErrorMessage?: string): v.InferOutput<Schema>;
declare function valibotRecursiveFlattenIssues(issues: v.BaseIssue<unknown>[]): Record<string, unknown>;

/**
 * Combine multiple uri parts into a single uri taking into account slashes.
 *
 * @param parts the parts to combine
 * @returns the combined url
 */
declare function joinUriParts(base: string, parts: string[]): string;

type Simplify<T> = {
    [KeyType in keyof T]: T[KeyType];
} & {};
type Optional<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
type StringWithAutoCompletion<T extends string> = T | (string & {});
type OrPromise<T> = T | Promise<T>;

declare function getQueryParams(url: string): Record<string, string>;
declare function objectToQueryParams(object: Record<string, unknown>): InstanceType<typeof _URLSearchParams>;

declare enum ContentType {
    XWwwFormUrlencoded = "application/x-www-form-urlencoded",
    Json = "application/json",
    JwkSet = "application/jwk-set+json"
}
declare function isContentType(contentType: ContentType, value: string): boolean;
declare function isResponseContentType(contentType: ContentType, response: FetchResponse): boolean;

type ValibotFetcher = <Schema extends v.BaseSchema<any, any, any>>(schema: Schema, expectedContentType: ContentType, ...args: Parameters<Fetch>) => Promise<{
    response: Awaited<ReturnType<Fetch>>;
    result?: v.SafeParseResult<Schema>;
}>;
/**
 * The default fetcher used when no
 * fetcher is provided.
 */
declare const defaultFetcher: typeof fetch;
/**
 * Creates a `fetchWithValibot` function that takes in a schema of
 * the expected response, and the arguments to fetch.
 *
 * If you don't provide a fetcher in `createValibotFetcher()`,
 * we're falling back to the default fetcher.
 *
 * @example
 *
 * const fetchWithValibot = createValibotFetcher();
 *
 * const { response, data } = await fetchWithValibot(
 *   v.looseObject({
 *    format: v.string()
 *   }),
 *   "https://example.com",
 * );
 */
declare function createValibotFetcher(
/**
 * A fetcher function that returns a response, from which the data can be parsed
 */
fetcher?: typeof fetch): ValibotFetcher;

declare const vHttpsUrl: v.SchemaWithPipe<[v.StringSchema<undefined>, v.UrlAction<string, undefined>, v.CheckAction<string, "url must be an https:// url">]>;
declare const vInteger: v.SchemaWithPipe<[v.NumberSchema<undefined>, v.IntegerAction<number, undefined>]>;
declare const vHttpMethod: v.PicklistSchema<["GET", "POST", "PUT", "DELETE", "HEAD", "OPTIONS", "TRACE", "CONNECT", "PATCH"], undefined>;
type HttpMethod = v.InferOutput<typeof vHttpMethod>;

/**
 * Deep merge two objects.
 * @param target
 * @param ...sources
 */
declare function mergeDeep(target: unknown, ...sources: Array<unknown>): unknown;

/**
 * Only primitive types allowed
 * Must have not duplicate entries (will always return false in this case)
 */
declare function arrayEqualsIgnoreOrder<Item extends string | number | boolean>(a: Array<Item>, b: Array<Item>): boolean;

interface WwwAuthenticateHeaderChallenge {
    scheme: string;
    /**
     * Record where the keys are the names, and the value can be 0 (null), 1 (string) or multiple (string[])
     * entries
     */
    payload: Record<string, string | string[] | null>;
}
declare function parseWwwAuthenticateHeader(str: string): WwwAuthenticateHeaderChallenge[];
declare function encodeWwwAuthenticateHeader(challenges: WwwAuthenticateHeaderChallenge[]): string;

interface Oid4vcTsConfig {
    /**
     * Whether to allow insecure http urls.
     *
     * @default false
     */
    allowInsecureUrls: boolean;
}
declare function setGlobalConfig(config: Oid4vcTsConfig): void;
declare function getGlobalConfig(): Oid4vcTsConfig;

export { type BaseSchema, ContentType, type Fetch, type FetchHeaders, type FetchRequestInit, type FetchResponse, _Headers as Headers, type HttpMethod, type InferOutputUnion, InvalidFetchResponseError, JsonParseError, type Oid4vcTsConfig, type Optional, type OrPromise, type Simplify, type StringWithAutoCompletion, _URL as URL, _URLSearchParams as URLSearchParams, type ValibotFetcher, ValidationError, type WwwAuthenticateHeaderChallenge, addSecondsToDate, arrayEqualsIgnoreOrder, createValibotFetcher, dateToSeconds, decodeBase64, decodeUtf8String, defaultFetcher, encodeToBase64, encodeToBase64Url, encodeToUtf8String, encodeWwwAuthenticateHeader, getGlobalConfig, getQueryParams, isContentType, isResponseContentType, joinUriParts, mergeDeep, objectToQueryParams, parseWithErrorHandling, parseWwwAuthenticateHeader, setGlobalConfig, stringToJsonWithErrorHandling, vHttpMethod, vHttpsUrl, vInteger, valibotRecursiveFlattenIssues };
